#include <stdio.h>
#include <stdlib.h>
#include <stdlib.h>
#include <signal.h>
#include <getopt.h>
#include <sys/timeb.h>
#include "obr.h"

#define PGROUND_X       25
#define PGROUND_Y       20

#define NR_MINES        150

/*
0..8	nr. of surrounding mines
9	mine
*/
#define F_MINE          9

unsigned char *pground=NULL;
int size_x, size_y, nr_mines;
int fl_debug = 0;

#define B_UNCOVERED     0x10
#define B_MARKED        0x20

#define COUNT_MASK      0x0F
#define COUNT(a)        ((a) & COUNT_MASK)

#define IS_UNCOVERED(a) ((a) & B_UNCOVERED)
#define IS_COVERED(a)   (!IS_UNCOVERED(a))

#define IS_MARKED(a)    ((a) & B_MARKED)

#define MARK(a)         (a) |= B_MARKED;
#define UNMARK(a)       (a) &= ~B_MARKED;

#define UNCOVER(a)      (a) |= B_UNCOVERED;

#define XY(x,y)         pground[(x) + (y) * size_x]

void
drawfield (int x, int y, int what)
{
  int w, att = GREY_COLOR;

#define DRAWCH(c)	mvwaddch(w_playground, y, x, (c))

  x += (PGWINSZ_X - size_x) >> 1;
  y += (PGWINSZ_Y - size_y) >> 1;

  if (!IS_UNCOVERED (what))
    {				/* '*' == hidden field */
      wattrset (w_playground, IS_MARKED (what) ? BRIGHTMAGENTA_COLOR
		: GREY_COLOR);
      DRAWCH ('*');
      return;
    }

/* extract mine info */
  w = COUNT (what);

  if (w == F_MINE)
    {
      wattrset (w_playground, BRIGHTRED_COLOR | A_BLINK);
      DRAWCH ('m');
      return;
    }

  if (!w)
    {
      wattrset (w_playground, GREY_COLOR);
      DRAWCH ('.');
      return;
    }

  switch (w)
    {
    case 1:
      att = BLUE_COLOR;
      break;
    case 2:
      att = BRIGHTGREEN_COLOR;
      break;
    case 3:
      att = BRIGHTRED_COLOR;
      break;
    case 4:
      att = BRIGHTRED_COLOR;
    case 5:
    case 6:
    case 7:
    case 8:
      att = RED_COLOR;
      break;

    }

  wattrset (w_playground, att);
  DRAWCH (w + '0');

#undef DRAWCH
}

void
place_cursor (int x, int y)
{
  wmove (w_playground, y + ((PGWINSZ_Y - size_y) >> 1),
	 x + ((PGWINSZ_X - size_x) >> 1));
}

/* null fields unhider */
void
zeroafter909 (int x, int y)
{
  UNCOVER (XY (x, y));
  drawfield (x, y, XY (x, y));

  if (x > 0 && !XY (x - 1, y))
    zeroafter909 (x - 1, y);

  if (x < size_x - 1 && !XY (x + 1, y))
    zeroafter909 (x + 1, y);

  if (y > 0 && !XY (x, y - 1))
    zeroafter909 (x, y - 1);

  if (y < size_y - 1 && !XY (x, y + 1))
    zeroafter909 (x, y + 1);
}

void
do_generate ()
{
  int i, cnt, r, todo, fields, mns = nr_mines;

  todo = fields = size_x * size_y;

  memset(pground, 0, size_x*size_y);

/* generate random mines placing */
  while (mns--)
    {
      r = (int) ((float) rand () * todo / (RAND_MAX + 1.0));
      i = 0;
      cnt = 0;
      while (i < fields)
	{
	  if (!pground[i])
	    {
	      if (cnt == r)
		{
		  pground[i] = F_MINE;
		  break;
		}
	      cnt++;
	    }
	  i++;
	}
    }

  /* count values for non-mine fields */
  for (i = 0; i < fields; i++)
    {
      if (!pground[i])
	{
	  int y = i / size_x, x = i % size_x;

	  /* up */
	  if (y)
	    {
	      if (pground[i - size_x] == F_MINE)
		pground[i]++;
	      /* and left */
	      if (x - 1 >= 0 && pground[i - size_x - 1] == F_MINE)
		pground[i]++;
	      /* and right */
	      if (x + 1 < size_x && pground[i - size_x + 1] == F_MINE)
		pground[i]++;
	    }

	  /* down */
	  if (y != size_y - 1)
	    {
	      if (pground[i + size_x] == F_MINE)
		pground[i]++;
	      /* and left */
	      if (x - 1 >= 0 && pground[i + size_x - 1] == F_MINE)
		pground[i]++;
	      /* and right */
	      if (x + 1 < size_x && pground[i + size_x + 1] == F_MINE)
		pground[i]++;
	    }

	  /* left */
	  if (x && pground[i - 1] == F_MINE)
	    pground[i]++;

	  /* right */
	  if (x != size_x - 1 && pground[i + 1] == F_MINE)
	    pground[i]++;
	}
    }
}

#define GAME_CONT	1
#define GAME_STOP	2

void
do_play ()
{
  int i, j, k, exit_code = 0;
  int cur_x, cur_y, nr_safe = size_x * size_y - nr_mines, nr_uncovered = 0;

  for (;;)
    {
      do_generate();

      for (j = 0; j < size_y; j++)
	{
	  for (i = 0; i < size_x; i++)
	    drawfield (i, j, XY (i, j));
	}

      cur_x = 0;
      cur_y = 0;
      place_cursor (cur_x, cur_y);

      refresh ();
      wrefresh (w_playground);

      do
	{
	  k = getch ();

	  switch (k)
	    {
	    case KEY_UP:
	    case 'q':		/* up */
	      if (cur_y > 0)
		cur_y--;
	      break;

	    case KEY_DOWN:
	    case 'a':		/* down */
	      if (cur_y < size_y - 1)
		cur_y++;
	      break;

	    case KEY_LEFT:
	    case 'o':		/* left */
	      if (cur_x > 0)
		cur_x--;
	      break;

	    case KEY_RIGHT:
	    case 'p':		/* right */
	      if (cur_x < size_x - 1)
		cur_x++;
	      break;

	    case ' ':
	      /* uncover */
	      if (IS_UNCOVERED (XY (cur_x, cur_y)))
		break;

	      if (!COUNT (XY (cur_x, cur_y)))
		zeroafter909 (cur_x, cur_y);
	      else
		{
		  UNCOVER (XY (cur_x, cur_y));
		  drawfield (cur_x, cur_y, XY (cur_x, cur_y));
		  nr_uncovered++;

		  if (!fl_debug)
		    {
		      if (COUNT (XY (cur_x, cur_y)) == F_MINE)
			exit_code =
			  obr_askcontinue
			  ("You have lost the match!\n"
			   "  Another game? (Y/n)", GAME_CONT, GAME_STOP);
		      else
			{
			  if (nr_uncovered == nr_safe)
			    exit_code =
			      obr_askcontinue
			      ("You have won the match!\n"
			       " Another game? (Y/n)", GAME_CONT, GAME_STOP);
			}
		    }
		}
	      break;

	    case 12:		/* ^L */
	      erase ();
	      refresh ();
	      touchwin (w_playground);
	      touchwin (w_info);
	      wrefresh (w_playground);
	      wrefresh (w_info);
	      break;

	    case 's':		/* show all */
	      if (!fl_debug)
		break;

	      for (i = 0; i < size_x * size_y; i++)
		UNCOVER (pground[i]);

	      for (j = 0; j < size_y; j++)
		{
		  for (i = 0; i < size_x; i++)
		    drawfield (i, j, XY (i, j));
		}

	      break;

	    case 0x0A:		/* enter */
/* mark */
	      XY (cur_x, cur_y) ^= B_MARKED;
	      drawfield (cur_x, cur_y, XY (cur_x, cur_y));
	      break;
	    }

	  place_cursor (cur_x, cur_y);
	  wrefresh (w_playground);
	}
      while (k != 'x' && !exit_code);

      if (exit_code != GAME_CONT)
	return;

      exit_code = 0;
    }
}

void
minyend (int ret)
{
  obr_done ();

  if(pground!=NULL)
    free (pground);
    
  exit(ret);
}

void
sighandler (int signr)
{
  switch (signr)
    {
    case SIGINT:
      minyend (0);
    }
}

void
usage ()
{
  printf
    ("usage: miny --help (gives this screen) --sizex=XXX --sizey=YYY --mines=MMM\n");

  printf ("short options: miny -h -x XXX -y YYY -m MMM\n");
}


int
main (int argc, char **argv)
{
  char lbuff[80];
  int option_index = 0, i;
  struct option options[] = {
    {"help", 0, NULL, 'h'},
    {"sizex", 1, NULL, 'x'},
    {"sizey", 1, NULL, 'y'},
    {"mines", 1, NULL, 'm'},
    {"debug", 0, NULL, 'd'},
    {NULL,}
  };

  size_x = PGROUND_X;
  size_y = PGROUND_Y;
  nr_mines = NR_MINES;

  printf ("MineZ 1.0 (c) 2001, 2002 Phantom Development Team\n");

/* maximal map size */
#define MAX_X  75
#define MAX_Y  20

  opterr = 0;

  while ((i = getopt_long (argc, argv, "m:x:y:hd", options, &option_index)) !=
	 -1)
    {
      switch (i)
	{
	case 'h':
	  usage ();
	  return 0;

	case 'x':
	  size_x = atoi (optarg);
	  if (size_x <= 1)
	    size_x = 2;
	  else if (size_x > MAX_X)
	    size_x = MAX_X - 1;
	  break;

	case 'y':
	  size_y = atoi (optarg);
	  if (size_y <= 1)
	    size_y = 2;
	  else if (size_y > MAX_Y)
	    size_y = MAX_Y - 1;
	  break;

	case 'm':
	  nr_mines = atoi (optarg);
	  if (!nr_mines)
	    nr_mines = 1;
	  break;

	case 'd':
	  fl_debug = 1;
	  break;

	case '?':
	  printf ("invalid syntax\n");
	  usage ();
	  return -5;
	  break;

	default:
	  printf ("getopt returns 0x%x %d %c\n", i, i, i);
	  return -3;
	}
    }

  if (nr_mines >= size_x * size_y)
    nr_mines = size_x * size_y - 1;

  obr_init ();

  sprintf (lbuff, "Mines: %d  Xsize: %d  Xsize: %d  Fields: %d", 
  nr_mines, size_x, size_y, size_x * size_y);
  
  mvwaddstr (w_info, 0, 0, lbuff);
  refresh ();
  wrefresh (w_info);

  if ((pground = (unsigned char *) malloc (PGROUND_X * PGROUND_Y)) == NULL)
    {
      printf ("Out of memory :)))\n");
      return -1;
    }

  signal (SIGINT, &sighandler);

  memset (pground, 0, PGROUND_X * PGROUND_Y);

/* randomize... */
  {
    struct timeb tb;
    ftime (&tb);
    srand ((tb.time << 8) | tb.millitm);
  }

  do_play ();

  minyend (0);
}
