#include <string.h>
#include "obr.h"

WINDOW *w_playground, *w_info;

int
obr_init ()
{
  initscr ();
  cbreak ();
  noecho ();

#ifdef NCURSES_VERSION
  if (has_colors ())
    {
      start_color ();
      init_pair (1, COLOR_WHITE, COLOR_BLACK);
      init_pair (2, COLOR_CYAN, COLOR_BLUE);
      init_pair (3, COLOR_CYAN, COLOR_BLACK);
      init_pair (4, COLOR_GREEN, COLOR_BLACK);
      init_pair (5, COLOR_RED, COLOR_BLACK);
      init_pair (6, COLOR_MAGENTA, COLOR_BLACK);
      init_pair (7, COLOR_YELLOW, COLOR_BLACK);
    }
#endif

  w_playground = newwin (LINES - 1, COLS, 1, 0);
  w_info = newwin (1, COLS, 0, 0);

  wattrset (w_playground, BRIGHTBLUE_COLOR);
  werase (w_playground);

  keypad (stdscr, TRUE);

  wattrset (w_info, INFO_COLOR);
  wbkgd (w_info, INFO_COLOR);
  werase (w_info);

  return 0;
}

int
obr_askcontinue (char *msg, int s, int f)
{
  int flen;
  char *nl;
  WINDOW *banner;

#define WX 30
#define WY 4

  banner = newwin (WY, WX, (LINES - WY) >> 1, (COLS - WX) >> 1);
  if (banner == NULL)
    return f;

  wattrset (banner, INFO_COLOR);
  wbkgd (banner, INFO_COLOR);
  werase (banner);

  nl = strchr (msg, '\n');
  if (nl == NULL)
    flen = strlen (msg);
  else
    {
      flen = nl - msg;
    }

  mvwaddnstr (banner, 1, (WX - flen) >> 1, msg, flen);
  if(nl!=NULL)
    mvwaddstr (banner, 2, (WX - flen) >> 1, nl+1);
    
  wrefresh (banner);

  for (;;)
    {
      switch (getch ())
	{
	case 0x0A:
	case 'y':
	case 'Y':
	  touchwin(w_playground);
	  wrefresh(w_playground);
	  return s;

	case 'n':
	case 'N':
	  touchwin(w_playground);
	  wrefresh(w_playground);
	  return f;
	}
    }

#undef WX
#undef WY

}

int
obr_done ()
{
  wattrset (w_playground, DEFAULT_COLOR);
  wbkgd (w_playground, DEFAULT_COLOR);
  werase (w_playground);
  wrefresh (w_playground);
  wattrset (w_info, DEFAULT_COLOR);
  wbkgd (w_info, DEFAULT_COLOR);
  werase (w_info);
  wrefresh (w_info);

  curs_set (TRUE);

  delwin (w_info);
  delwin (w_playground);
  endwin ();

  return 0;
}
